/*
 * Copyright (c) 2002, 2003, 2004
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that: (1) source code distributions
 * retain the above copyright notice and this paragraph in its entirety, (2)
 * distributions including binary code include the above copyright notice and
 * this paragraph in its entirety in the documentation or other materials
 * provided with the distribution, and (3) all advertising materials mentioning
 * features or use of this software display the following acknowledgement:
 * ``This product includes software developed by the University of California,
 * Lawrence Berkeley Laboratory and its contributors.'' Neither the name of
 * the University nor the names of its contributors may be used to endorse
 * or promote products derived from this software without specific prior
 * written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef lint
static const char rcsid[] =
    "@(#) $Id: acl.c,v 1.21 2004/08/09 20:30:16 leres Exp $ (LBL)";
#endif

#include <sys/types.h>
#include <sys/time.h>

#include <ctype.h>
#ifdef HAVE_MALLOC_H
#include <malloc.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sysexits.h>
#include <syslog.h>

#include "acld.h"
#include "cf.h"
#include "util.h"

/* Locals */
static struct s2v str2acl[] = {
	{ "notconnected",	ATYPE_UNKNOWN },

	{ "permitany",		ATYPE_PERMITANY },
	{ "blockany",		ATYPE_BLOCKANY },

	{ "permithost",		ATYPE_PERMITHOST },
	{ "blockhost",		ATYPE_BLOCKHOST },

	{ "permitnet",		ATYPE_PERMITNET },
	{ "blocknet",		ATYPE_BLOCKNET },

	{ "permitudpport",	ATYPE_PERMITUDPPORT },
	{ "blockudpport",	ATYPE_BLOCKUDPPORT },
	{ "permittcpport",	ATYPE_PERMITTCPPORT },
	{ "blocktcpport",	ATYPE_BLOCKTCPPORT },

	{ "permitudphostport",	ATYPE_PERMITUDPHOSTPORT },
	{ "blockudphostport",	ATYPE_BLOCKUDPHOSTPORT },
	{ "permittcphostport",	ATYPE_PERMITTCPHOSTPORT },
	{ "blocktcphostport",	ATYPE_BLOCKTCPHOSTPORT },

	{ "permitudpdsthost",	ATYPE_PERMITUDPDSTHOST },
	{ "blockudpdsthost",	ATYPE_BLOCKUDPDSTHOST },
	{ "permittcpdsthost",	ATYPE_PERMITTCPDSTHOST },
	{ "blocktcpdsthost",	ATYPE_BLOCKTCPDSTHOST },

	{ "permitipdsthost",	ATYPE_PERMITIPDSTHOST },
	{ "blockipdsthost",	ATYPE_BLOCKIPDSTHOST },
	{ "permiticmpdsthost",	ATYPE_PERMITICMPDSTHOST },
	{ "blockicmpdsthost",	ATYPE_BLOCKICMPDSTHOST },

	{ "permitipdstnet",	ATYPE_PERMITIPDSTNET },
	{ "blockipdstnet",	ATYPE_BLOCKIPDSTNET },
	{ "permiticmpdstnet",	ATYPE_PERMITICMPDSTNET },
	{ "blockicmpdstnet",	ATYPE_BLOCKICMPDSTNET },

	{ "blockudpdsthostport", ATYPE_BLOCKUDPDSTHOSTPORT },
	{ "blocktcpdsthostport", ATYPE_BLOCKTCPDSTHOSTPORT },
	{ "permitudpdsthostport", ATYPE_PERMITUDPDSTHOSTPORT },
	{ "permittcpdsthostport", ATYPE_PERMITTCPDSTHOSTPORT },

	{ "permitudpnetport",	ATYPE_PERMITUDPNETPORT },
	{ "blockudpnetport",	ATYPE_BLOCKUDPNETPORT },
	{ "permittcpnetport",	ATYPE_PERMITTCPNETPORT },
	{ "blocktcpnetport",	ATYPE_BLOCKTCPNETPORT },

	{ "permitudphostsrcport", ATYPE_PERMITUDPHOSTSRCPORT },
	{ "blockudphostsrcport", ATYPE_BLOCKUDPHOSTSRCPORT },
	{ "permittcphostsrcport", ATYPE_PERMITTCPHOSTSRCPORT },
	{ "blocktcphostsrcport", ATYPE_BLOCKTCPHOSTSRCPORT },

	{ "permitudphostpairdstport", ATYPE_PERMITUDPHOSTPAIRDSTPORT },
	{ "blockudphostpairdstport", ATYPE_BLOCKUDPHOSTPAIRDSTPORT },
	{ "permittcphostpairdstport", ATYPE_PERMITTCPHOSTPAIRDSTPORT },
	{ "blocktcphostpairdstport", ATYPE_BLOCKTCPHOSTPAIRDSTPORT },

	{ NULL,			0 }
};

/* Forwards */
int aclcmp(const void *, const void *);
int acllistcmp(const void *, const void *);
int addrcmp(const void *, const void *);

void
acladdacl(register struct acllist *ap, register struct acl *nal)
{
	register struct acl *al;

	dynarray((void **)&ap->acl, &ap->acllen, &ap->aclsize,
	    sizeof(*ap->acl), 1024, 1024, "acl array");

	al = ap->acl + ap->acllen;
	*al = *nal;
	++ap->acllen;

	/* Sort on sequence number (if we have them) */
	if (ap->lastportseq > 0 || ap->lastseq > 0)
		qsort(ap->acl, (size_t)ap->acllen, sizeof(*al), aclcmp);
}

void
acladdacllist(register struct cf *cf, register const char *str)
{
	register const char *p;
	register struct acllist *ap;
	register struct addr *a;
	register int an;
	char **av;

	an = makeargv(str, &av);
	if (an < 2 || an > 3) {
		log(LOG_ERR, "acladdacllist wrong number of args: \"%s\"", str);
		freeargv(av);
		return;
	}
	ap = aclfindlistbyname(cf, av[0]);
	if (ap == NULL) {
		dynarray((void **)&cf->c_acllist, &cf->c_acllistlen,
		    &cf->c_acllistsize, sizeof(*cf->c_acllist),
		    1, 1, "cf acllist");
		ap = cf->c_acllist + cf->c_acllistlen;
		ap->name = strsave(av[0]);
		++cf->c_acllistlen;

	}
	dynarray((void **)&ap->addr, &ap->addrlen, &ap->addrsize,
	    sizeof(*ap->addr), 1, 1, "acllist addrs");

	a = ap->addr + ap->addrlen;
	p = extractaddr(av[1], an == 3 ? av[2] : NULL, a);
	if (p != NULL) {
		log(LOG_ERR, "acladdacllist extractaddr: %s", p);
		freeargv(av);
		return;
	}
	++ap->addrlen;
	freeargv(av);
}

int
aclcmp(register const void *arg1, register const void *arg2)
{
	register const struct acl *al1, *al2;

	al1 = (const struct acl *)arg1;
	al2 = (const struct acl *)arg2;
	return (al1->seq - al2->seq);
}

int
acldeleteacl(register struct acllist *ap, register struct acl *oal)
{
	register struct acl *al;
	register int i, n;

	for (i = 0, al = ap->acl; i < ap->acllen; ++i, ++al)
		if (al->type == oal->type &&
		    al->addr1.addr == oal->addr1.addr &&
		    al->addr1.mask == oal->addr1.mask)
			break;

	if (i >= ap->acllen)
		return (0);

	if (al->port1 != NULL) {
		free(al->port2);
		al->port2 = NULL;
	}
	if (al->port2 != NULL) {
		free(al->port2);
		al->port2 = NULL;
	}
	if (al->raw != NULL) {
		free(al->raw);
		al->raw = NULL;
	}

	n = ap->acllen - (i + 1);
	memmove(al, al + 1, n * sizeof(*al));
	memset(ap->acl + ap->acllen, 0, sizeof(*ap->acl));
	--ap->acllen;
	return (1);
}

struct acl *
aclfindacl(register struct acllist *ap, register struct acl *al)
{
	register struct acl *al2;
	register int i;

	for (i = 0, al2 = ap->acl; i < ap->acllen; ++i, ++al2)
		if (al->type == al2->type) {
			switch (al->type) {

			case ATYPE_BLOCKHOST:
				if (memcmp(&al->addr1, &al2->addr1,
				    sizeof(al->addr1)) == 0)
					return (al2);
				break;

			case ATYPE_BLOCKUDPPORT:
			case ATYPE_BLOCKTCPPORT:
				if (strcasecmp(al->port1, al2->port1) == 0)
					return (al2);
				break;

			default:
				break;
			}
		}
	return (NULL);
}

struct acllist *
aclfindlistbyaddr(register struct cf *cf, register struct addr *a)
{
	register struct acllist *ap;
	register struct addr *a2;
	register int i, j;

	for (i = 0, ap = cf->c_acllist; i < cf->c_acllistlen; ++i, ++ap)
		for (j = 0, a2 = ap->addr; j < ap->addrlen; ++j, ++a2) {
			if (ISDEFAULTADDR(a2))
				return (ap);
			if (a2->addr == (a2->mask & a->addr))
				return (ap);
		}
	return (NULL);
}

struct acllist *
aclfindlistbyname(register struct cf *cf, register const char *name)
{
	register struct acllist *ap;
	register int i;

	for (i = 0, ap = cf->c_acllist; i < cf->c_acllistlen; ++i, ++ap)
		if (strcmp(ap->name, name) == 0)
			return (ap);
	return (NULL);
}

const char *
aclformat(register struct acl *al)
{
	register const char *p;
	register char *cp;
	register size_t size;
	static char buf[1024];

	p = val2str(str2acl, al->type);
	cp = buf;
	size = sizeof(buf);
	(void)snprintf(cp, size, "%d %d %s", al->seq, al->count, p);
	size -= strlen(cp);
	cp += strlen(cp);

	switch (al->type) {

	case ATYPE_PERMITANY:
	case ATYPE_BLOCKANY:
		break;

	case ATYPE_PERMITHOST:
	case ATYPE_BLOCKHOST:
	case ATYPE_PERMITUDPDSTHOST:
	case ATYPE_BLOCKUDPDSTHOST:
	case ATYPE_PERMITTCPDSTHOST:
	case ATYPE_BLOCKTCPDSTHOST:
	case ATYPE_PERMITIPDSTHOST:
	case ATYPE_BLOCKIPDSTHOST:
	case ATYPE_PERMITICMPDSTHOST:
	case ATYPE_BLOCKICMPDSTHOST:
		(void)snprintf(cp, size, " %s", addr2str(al->addr1.addr));
		break;

	case ATYPE_PERMITNET:
	case ATYPE_BLOCKNET:
	case ATYPE_PERMITIPDSTNET:
	case ATYPE_BLOCKIPDSTNET:
	case ATYPE_PERMITICMPDSTNET:
	case ATYPE_BLOCKICMPDSTNET:
		(void)snprintf(cp, size, " %s/%d",
		    addr2str(al->addr1.addr), maskwidth(al->addr1.mask));
		break;

	case ATYPE_PERMITUDPPORT:
	case ATYPE_BLOCKUDPPORT:
	case ATYPE_PERMITTCPPORT:
	case ATYPE_BLOCKTCPPORT:
		(void)snprintf(cp, size, " %s", al->port1);
		break;

	case ATYPE_PERMITUDPHOSTPORT:
	case ATYPE_BLOCKUDPHOSTPORT:
	case ATYPE_PERMITTCPHOSTPORT:
	case ATYPE_BLOCKTCPHOSTPORT:
	case ATYPE_PERMITUDPDSTHOSTPORT:
	case ATYPE_BLOCKUDPDSTHOSTPORT:
	case ATYPE_PERMITTCPDSTHOSTPORT:
	case ATYPE_BLOCKTCPDSTHOSTPORT:
	case ATYPE_PERMITUDPHOSTSRCPORT:
	case ATYPE_BLOCKUDPHOSTSRCPORT:
	case ATYPE_PERMITTCPHOSTSRCPORT:
	case ATYPE_BLOCKTCPHOSTSRCPORT:
		(void)snprintf(cp, size, " %s", addr2str(al->addr1.addr));
		size -= strlen(cp);
		cp += strlen(cp);
		(void)snprintf(cp, size, " %s", al->port1);
		break;

	case ATYPE_PERMITUDPNETPORT:
	case ATYPE_BLOCKUDPNETPORT:
	case ATYPE_PERMITTCPNETPORT:
	case ATYPE_BLOCKTCPNETPORT:
		(void)snprintf(cp, size, " %s/%d",
		    addr2str(al->addr1.addr), maskwidth(al->addr1.mask));
		size -= strlen(cp);
		cp += strlen(cp);
		(void)snprintf(cp, size, " %s", al->port1);
		break;

	case ATYPE_PERMITUDPHOSTPAIRDSTPORT:
	case ATYPE_BLOCKUDPHOSTPAIRDSTPORT:
	case ATYPE_PERMITTCPHOSTPAIRDSTPORT:
	case ATYPE_BLOCKTCPHOSTPAIRDSTPORT:
		(void)snprintf(cp, size, " %s", addr2str(al->addr1.addr));
		size -= strlen(cp);
		cp += strlen(cp);

		(void)snprintf(cp, size, " %s", addr2str(al->addr2.addr));
		size -= strlen(cp);
		cp += strlen(cp);

		(void)snprintf(cp, size, " %s", al->port1);
		break;

	case ATYPE_UNKNOWN:
		(void)snprintf(buf, sizeof(buf), "# %d unsupported acl \"%s\"",
		    al->seq, al->raw != NULL ? al->raw : "");
		break;

	default:
		log(LOG_ERR, "aclformat: bad type %d", al->type);
		exit(EX_SOFTWARE);
	}
	return (buf);
}

void
aclfree(register struct acllist *ap)
{
	register struct acl *al;
	register struct req *rp;
	register int i;

	if (ap->name != NULL) {
		free(ap->name);
		ap->name = NULL;
	}

	/* Compaction cleanup */
	ap->compactclient = NULL;
	if ((rp = ap->compactreq) != NULL) {
		ap->compactreq = NULL;
		freereq(rp);
	}

	if (ap->acl != NULL) {
		for (i = 0, al = ap->acl; i < ap->acllen; ++i, ++al) {
			if (al->port1 != NULL)
				free(al->port1);
			if (al->port2 != NULL)
				free(al->port2);
			if (al->raw != NULL)
				free(al->raw);
		}
		freedynarray((void **)&ap->acl, &ap->acllen, &ap->aclsize);
	}
	if (ap->addr != NULL)
		freedynarray((void **)&ap->addr, &ap->addrlen, &ap->addrsize);
	ap->sentlistacl = 0;
}

int
acllistcmp(register const void *arg1, register const void *arg2)
{
	register const struct acllist *ap1, *ap2;

	ap1 = (const struct acllist *)arg1;
	ap2 = (const struct acllist *)arg2;
	/* Sort on the first address (good enough to move default to the end) */
	return (addrcmp(ap1->addr, ap2->addr));
}

/* Free just the acl entries from all acl lists */
void
acllistsfree(register struct state  *sp)
{
	register int i, j;
	register struct acllist *ap;
	register struct acl *al;
	register struct cf *cf;

	cf = sp->cf;
	for (i = 0, ap  = cf->c_acllist; i < cf->c_acllistlen; ++i, ++ap) {
		if (ap->acl == NULL)
			continue;
		for (j = 0, al = ap->acl; j < ap->acllen; ++j, ++al) {
			if (al->port1 != NULL)
				free(al->port1);
			if (al->port2 != NULL)
				free(al->port2);
		}
		freedynarray((void **)&ap->acl, &ap->acllen, &ap->aclsize);
		ap->sentlistacl = 0;
	}
	sp->listedallacls = 0;
}

void
aclsortacls(register struct cf *cf)
{
	register struct acllist *ap;
	register struct addr *a;
	register int i;

	ap = cf->c_acllist;
	qsort(ap, (size_t)cf->c_acllistlen, sizeof(*ap), acllistcmp);
	for (i = 0, ap = cf->c_acllist; i < cf->c_acllistlen; ++i, ++ap) {
		if (ap->addrlen > 1) {
			a = ap->addr;
			qsort(a, (size_t)ap->addrlen, sizeof(*a), addrcmp);
		}
		if (cf->c_lowportseq > 0 && ap->lastportseq < cf->c_lowportseq)
			ap->lastportseq = cf->c_lowportseq - 1;
		if (cf->c_incrseq) {
			if (cf->c_lowseq > 0 && ap->lastseq < cf->c_lowseq)
				ap->lastseq = cf->c_lowseq - 1;
		} else {
			if (cf->c_highseq > 0 &&
			    (ap->lastseq == 0 || ap->lastseq > cf->c_highseq))
				ap->lastseq = cf->c_highseq + 1;
		}
	}
}

int
aclstraddacl(register struct cf *cf, register struct acllist *ap,
    register const char *str)
{
	register struct acl *al;
	register const char *p;
	struct acl acl;
	register int an;
	char **av;

	/* Eat leading dot; done if there was only one */
	if (*str == '.') {
		++str;
		if (*str == '\n'|| *str == '\0')
			return (1);
	}

	/* syslog any comments we get */
	if (*str == '#') {
		++str;
		while (*str == ' ')
			++str;
		log(LOG_INFO, "listacl: \"%s\"", str);
		return (0);
	}


	al = &acl;
	memset(al, 0, sizeof(*al));
	an = makeargv(str, &av);

	if (an < 3) {
		log(LOG_ERR, "aclstraddacl: missing args \"%s\"", str);
		freeargv(av);
		return(0);
	}

	al->seq = atoi(av[0]);
	al->count = atoi(av[1]);
	al->type = str2val(str2acl, av[2]);
	switch (al->type) {

	case ATYPE_PERMITANY:
	case ATYPE_BLOCKANY:
		if (an != 3) {
			log(LOG_ERR,
			    "aclstraddacl: wrong number of args \"%s\"", str);
			freeargv(av);
			return(0);
		}
		break;

	case ATYPE_PERMITHOST:
	case ATYPE_BLOCKHOST:
	case ATYPE_PERMITUDPDSTHOST:
	case ATYPE_BLOCKUDPDSTHOST:
	case ATYPE_PERMITTCPDSTHOST:
	case ATYPE_BLOCKTCPDSTHOST:
	case ATYPE_PERMITIPDSTHOST:
	case ATYPE_BLOCKIPDSTHOST:
	case ATYPE_PERMITICMPDSTHOST:
	case ATYPE_BLOCKICMPDSTHOST:
	case ATYPE_PERMITNET:
	case ATYPE_BLOCKNET:
	case ATYPE_PERMITIPDSTNET:
	case ATYPE_BLOCKIPDSTNET:
	case ATYPE_PERMITICMPDSTNET:
	case ATYPE_BLOCKICMPDSTNET:
		if (an != 4) {
			log(LOG_ERR,
			    "aclstraddacl: wrong number of args \"%s\"", str);
			freeargv(av);
			return(0);
		}
		p = extractaddr(av[3], NULL, &al->addr1);
		if (p != NULL) {
			log(LOG_ERR, "aclstraddacl: bad addr: %s", p);
			freeargv(av);
			return(0);
		}
		break;

	case ATYPE_PERMITUDPPORT:
	case ATYPE_BLOCKUDPPORT:
	case ATYPE_PERMITTCPPORT:
	case ATYPE_BLOCKTCPPORT:
		if (an != 4) {
			log(LOG_ERR,
			    "aclstraddacl: wrong number of args \"%s\"", str);
			freeargv(av);
			return(0);
		}
		al->port1 = strsave(av[3]);
		break;

	case ATYPE_PERMITUDPHOSTPORT:
	case ATYPE_BLOCKUDPHOSTPORT:
	case ATYPE_PERMITTCPHOSTPORT:
	case ATYPE_BLOCKTCPHOSTPORT:
	case ATYPE_PERMITUDPDSTHOSTPORT:
	case ATYPE_BLOCKUDPDSTHOSTPORT:
	case ATYPE_PERMITTCPDSTHOSTPORT:
	case ATYPE_BLOCKTCPDSTHOSTPORT:
	case ATYPE_PERMITUDPHOSTSRCPORT:
	case ATYPE_BLOCKUDPHOSTSRCPORT:
	case ATYPE_PERMITTCPHOSTSRCPORT:
	case ATYPE_BLOCKTCPHOSTSRCPORT:
		if (an != 5) {
			log(LOG_ERR,
			    "aclstraddacl: wrong number of args \"%s\"", str);
			freeargv(av);
			return(0);
		}
		p = extractaddr(av[3], NULL, &al->addr1);
		if (p != NULL) {
			log(LOG_ERR, "aclstraddacl: bad addr: %s", p);
			freeargv(av);
			return(0);
		}
		al->port1 = strsave(av[4]);
		break;

	case ATYPE_PERMITUDPNETPORT:
	case ATYPE_BLOCKUDPNETPORT:
	case ATYPE_PERMITTCPNETPORT:
	case ATYPE_BLOCKTCPNETPORT:
		if (an != 5) {
			log(LOG_ERR,
			    "aclstraddacl: wrong number of args \"%s\"", str);
			freeargv(av);
			return(0);
		}
		p = extractaddr(av[3], NULL, &al->addr1);
		if (p != NULL) {
			log(LOG_ERR, "aclstraddacl: bad net: %s", p);
			freeargv(av);
			return(0);
		}
		al->port1 = strsave(av[4]);
		break;

	case ATYPE_PERMITUDPHOSTPAIRDSTPORT:
	case ATYPE_BLOCKUDPHOSTPAIRDSTPORT:
	case ATYPE_PERMITTCPHOSTPAIRDSTPORT:
	case ATYPE_BLOCKTCPHOSTPAIRDSTPORT:
		if (an != 6) {
			log(LOG_ERR,
			    "aclstraddacl: wrong number of args \"%s\"", str);
			freeargv(av);
			return(0);
		}
		p = extractaddr(av[3], NULL, &al->addr1);
		if (p != NULL) {
			log(LOG_ERR, "aclstraddacl: bad addr1: %s", p);
			freeargv(av);
			return(0);
		}
		p = extractaddr(av[4], NULL, &al->addr2);
		if (p != NULL) {
			log(LOG_ERR, "aclstraddacl: bad addr2: %s", p);
			freeargv(av);
			return(0);
		}
		al->port1 = strsave(av[5]);
		break;

	case ATYPE_UNKNOWN:
		/* Save the raw text so we can print it out later */
		p = strchr(str, '#');
		if (p == NULL)
			p = str;
		else {
			++p;
			while (isspace((int)*p))
				++p;
		}
		al->raw = strsave(p);
		log(LOG_ERR, "aclstraddacl: unknown \"%s\"", str);
		break;

	default:
		log(LOG_ERR, "aclstraddacl: no such type \"%s\"", str);
		exit(EX_SOFTWARE);
	}

	acladdacl(ap, al);

	if ((cf->c_highseq == 0 || al->seq <= cf->c_highseq) &&
	    (cf->c_lowseq == 0 || al->seq >= cf->c_lowseq)) {
		if (cf->c_incrseq) {
			if (ap->lastseq < al->seq)
				ap->lastseq = al->seq;
		} else {
			if (ap->lastseq == 0 || ap->lastseq > al->seq)
				ap->lastseq = al->seq;
		}
	}

	if ((cf->c_highportseq == 0 || al->seq <= cf->c_highportseq) &&
	    (cf->c_lowportseq == 0 || al->seq >= cf->c_lowportseq)) {
		if (ap->lastportseq < al->seq)
			ap->lastportseq = al->seq;
	}

	freeargv(av);
	return (0);
}

int
addrcmp(register const void *arg1, register const void *arg2)
{
	register const struct addr *a1, *a2;
	register int r1, r2;

	a1 = (const struct addr *)arg1;
	a2 = (const struct addr *)arg2;

	r1 = ISDEFAULTADDR(a1);
	r2 = ISDEFAULTADDR(a2);
	if (r1 || r2) {
		if (!r1 && r2)
			return (-1);
		else if (r1 && !r2)
			return (1);
		else
			return (0);
	}

	if (a1->addr < a2->addr)
		r1 = -1;
	else if (a1->addr > a2->addr)
		r1 = 1;
	else
		r1 = 0;

	if (a1->mask < a2->mask)
		r2 = 1;
	else if (a1->mask > a2->mask)
		r2 = -1;
	else
		r2 = 0;

	/* If the masks are the same, sort on addr */
	if (r2 == 0)
		return (r1);

	/* Otherwise, sort on mask */
	return (r2);
}
