#!/usr/local/bin/expect -f
# @(#) $Id: acl.sh,v 1.24 2004/11/19 00:39:08 leres Exp $ (LBL)
#
# acl.sh - talk to acld
#

set prog $argv0
set len [string length $prog]
for { set i $len } { $i > 0 } { set i [expr $i - 1] } {
	if { [string index $prog $i ] == "/" } {
		incr i
		set prog [string range $prog $i $len]
		break
	}
}

match_max 2048

set f "[string trimright $prog ".sh"].log"
if [catch {log_file -a  /usr/operator/acl2/logs/$f} err] {
	send_error "$prog: warning: $err\n"
	if [catch {log_file -a  $f} err] {
		send_error "$prog: warning: $err\n"
	}
}
log_user 0

if [catch {set ident [set env(USER)]} err] {
	set ident "whoknows"
}
if [catch {set host [set env(HOST)]} err] {
	if [catch {set host [set env(HOSTNAME)]} err] {
		set host ""
	}
}
if { $host != "" } {
	append ident "@$host"
}

#
# Print usage and exit
#
proc usage {} {
	global prog

	set c "\[-c comment ...\]"
	send_error "usage: $prog \[drop|restore|query] addr $c\n"
	send_error "       $prog \[drop|restore|query] -f file $c\n"
	send_error "       $prog \[dropudpport|restoreudpport] port ACL $c\n"
	send_error "       $prog \[droptcpport|restoretcpport] port ACL $c\n"
	send_error "       $prog listacl ACL $c\n"
	send_error "       $prog compact ACL $c\n"
	send_error "       $prog state $c\n"
	exit 1
}

#
# Reap response
#
proc getresponse { what cookie } {
	global prog
	global comment
	global lasttime

	set response "^(\[0-9]+\.\[0-9]+) $cookie $what"
	set doingcomment 0
	set comment ""
	set status 0
	set lasttime 0
	if { $what == "compact" || $what == "listacl" } {
		set timeout 3600
	} else {
		set timeout 90
	}
	expect {
		-re "$response\r\n" {
			set lasttime $expect_out(1,string)
			# done
		}
		-re "$response -\r\n" {
			set lasttime $expect_out(1,string)
			set doingcomment 1
			# done
		}
		-re "$response-failed\r\n" {
			set lasttime $expect_out(1,string)
			set status 1
			# done
		}
		-re "$response-failed -\r\n" {
			set lasttime $expect_out(1,string)
			set doingcomment 1
			set status 1
			# done
		}
		timeout {
			send_error "$prog: timeout $what ($timeout secs)\n"
			exit 1
		}
		eof {
			send_error "$prog: child exited\n"
			exit 1
		}
	}
	if { $doingcomment != 0 } {
		expect {
			"^.\r\n" {
				if { $lasttime == 0 } {
					send_error \
					    "$prog: missing lasttime at EOM\n"
					exit 1
				}
				# done
			}
			-re "^(\[^\r\n]*)\r\n" {
				append comment "$expect_out(1,string)\n"
				exp_continue
			}
			timeout {
				send_error \
				    "$prog: timeout2 $what ($timeout secs)\n"
				exit 1
			}
			eof {
				send_error "$prog: child exited2\n"
				exit 1
			}
		}
	}
	if { $comment != "" } {
		if { $status != 0 } {
			send_error "$prog: acld error: $comment"
		} elseif { $what != "acld" } {
			send_user "$comment"
		}
	}
	return $status
}

# Send acld commands with identification
proc sendcmd {line} {
	global ident
	global usercomment

	send -- "$line -\r"
	send -- "{$ident}\r"
	if { [string length $usercomment] > 0 } {
		send -- "$usercomment\r"
	}
	send -- ".\r"
}

#
# Main program
#
if { $argc < 1 } {
	usage
}

set what [lindex $argv 0]
set file ""
set addr ""
set port ""
set acl ""
set acl
if [catch {set aclipaddr [set env(ACLIPADDR)]} err] {
	set aclipaddr "127.0.0.1"
}
if [catch {set aclport [set env(ACLPORT)]} err] {
	set aclport "1965"
}

# Optional user comment (must trail due to the way expect processes arguments)
set usercomment ""
for { set i 0 } { $i < $argc } { set i [expr $i + 1] } {
	if { [lindex $argv $i] == "-c" } {
		set usercomment "[lrange $argv [expr $i + 1] [expr $argc - 1]]"
		if { $usercomment == "." } {
			set $usercomment ".$usercomment"
		}
		set argc $i
		break
	}
}

if { $what == "drop" || $what == "restore" || $what == "query" } {
	if { [lindex $argv 1] == "-f" } {
		if { $argc != 3 } {
			usage
		}
		set file [lindex $argv 2]
		if [catch {open $file} fd] {
			send_error "$prog: $fd\n"
			exit 1
		}
		set fd [open $file]
	} elseif { $argc == 2 } {
		set addr [lindex $argv 1]
	} else {
		usage
	}
} elseif { $what == "dropudpport" || $what == "droptcpport" ||
    $what == "restoreudpport" || $what == "restoretcpport" } {
	if { $argc != 3 } {
		usage
	}
	set port [lindex $argv 1]
	set acl [lindex $argv 2]
} elseif { $argc == 2 && ( $what == "compact" || $what == "listacl" ) } {
	if { $argc != 2 } {
		usage
	}
	set acl [lindex $argv 1]
} elseif { $what == "state" } {
	if { $argc != 1 } {
		usage
	}
} else {
	usage
}

send_error "$prog: [clock format [clock seconds] -format "%b %d %T"] $argv\n"

#
# Connect to the acl daemon
#
set stty_init "-echo"
spawn socket -c $aclipaddr $aclport
if { [getresponse "acld" 0] != 0 } {
	send_error "$prog: \"socket -c $aclipaddr $aclport\" failed\n"
	exit 1
}
set start $lasttime

set lastcookie 0
set firstcookie 1
set outstanding 0
set estatus 0
if { $what == "compact" || $what == "listacl" } {
	#
	# list/compact an ACL
	#
	incr lastcookie
	sendcmd "$what $lastcookie $acl"
	incr outstanding
} elseif { $what == "state" } {
	#
	# state inquiry
	#
	incr lastcookie
	sendcmd "$what $lastcookie"
	incr outstanding
} elseif { $addr != "" } {
	#
	# Handle a single address
	#
	incr lastcookie
	sendcmd "$what $lastcookie $addr"
	incr outstanding
} elseif { $port != "" } {
	#
	# Handle a port
	#
	incr lastcookie
	sendcmd "$what $lastcookie $port $acl"
	incr outstanding
} else {
	#
	# Loop on addrs from the file
	#
	set maxoutstanding 5
	while {[gets $fd addr] != -1} {
		incr lastcookie
		sendcmd "$what $lastcookie $addr"
		incr outstanding
		if { $outstanding >= $maxoutstanding } {
			if { [getresponse $what $firstcookie] != 0 } {
				set estatus 1
			}
			incr firstcookie
			set outstanding [expr $outstanding - 1]
		}
	}
}

# Finish up
while { $outstanding > 0 } {
	if { [getresponse $what $firstcookie] != 0 } {
		set estatus 1
	}
	incr firstcookie
	set outstanding [expr $outstanding - 1]
}

#send_error "(start $start finish $lasttime)\n"
set delta [format "%.3f" [expr $lasttime - $start]]

send_error "$prog: (took $delta seconds)\n"

incr lastcookie
send -- "exit $lastcookie\r"
expect "^(\[0-9]+\.\[0-9]+) $lastcookie exit"

if { $estatus != 0 } {
	send_error "$prog: $what FAILED\n"
}

exit $estatus
