/* @(#) $Id: acld.h,v 1.17 2004/08/08 18:28:00 leres Exp $ (LBL) */

#include "io.h"
#include "timer.h"
#include "client.h"

/* acld states */
enum acldstate {
	ASTATE_NOTCONNECTED = 0,
	ASTATE_CONNECTED,
	ASTATE_LOGGEDIN,
	ASTATE_SENTLOGIN,
	ASTATE_SENTLISTACL,
	ASTATE_READRESPONSE,
	ASTATE_READERROR,
	ASTATE_READACL,
};

/* State */
struct state {
	enum acldstate state;		/* ASTATE_* */
	const char *cmd;		/* command sent */
	int nfds;			/* number of select file descriptors */
	int s;				/* listen socket */
	int rfd;			/* expect child read pipe */
	int wfd;			/* expect child read pipe */
	pid_t pid;			/* child pid */
	u_int nclients;			/* next client number */
	int listedallacls;		/* have we listed all the acl's? */
	struct timeval timeout;		/* select timeout */
	struct iobuf rbuf;		/* child read buffer */
	struct req *req;		/* linked list of server requests */
	struct client *clients;		/* linked list of clients */
	struct client *reqclient;	/* client we did the last request for */
	struct cf *cf;			/* configuration */
	struct timer t_ayt;		/* timer for "ayt" */
	struct timer t_compact;		/* timer for "compact" */
	struct timer t_login;		/* timer for next login attempt */
	struct timer t_sync;		/* timer for next sync */
};

#include "acl.h"

struct intlist {
	char *i_acl;
	char *i_name;
};

/* Request types */
enum reqtype {
	REQ_UNKNOWN = 0,
	REQ_RELOAD,		/* kill expect child process & reload config */
	REQ_DROP,		/* block a host or subnet */
	REQ_RESTORE,		/* unblock a host or subnet */
	REQ_DROPUDPPORT,	/* block a udp port */
	REQ_RESTOREUDPPORT,	/* unblock a udp port */
	REQ_DROPTCPPORT,	/* block a tcp port */
	REQ_RESTORETCPPORT,	/* unblock a tcp port */
	REQ_STATE,		/* display current acld state */
	REQ_LISTACL,		/* list contains of specified acl */
	REQ_QUERY,		/* check for a host or subnet block */
	REQ_COMPACT,		/* schedule compaction of specified acl */
	REQ_TEST,		/* (undocumented) */
	REQ_EXIT,		/* terminate client session */
	REQ_HELP,		/* display brief help message */
	REQ_SYNC,		/* copy router config to nonvolatile memory */
	REQ_AYT,		/* check that router is still alive */
};

/* Request states */
enum reqstate {
	RSTATE_PENDING = 0,	/* not processed yet */
	RSTATE_READCOMMENT,	/* reading client comments */
	RSTATE_READRESPONSE,	/* reading child response */
	RSTATE_CHILD,		/* waiting for child to reply */
	RSTATE_DONE		/* ok to remove from queue */
};

/* Request */
struct req {
	struct req *next;
	enum reqtype type;		/* REQ_* */
	enum reqstate state;		/* RSTATE_* */
	const char *cmd;		/* client command name */
	char *aclname;			/* REQ_LISTACL ACL name */
	u_int32_t cookie;
	struct timeval ats;		/* arrival timestamp */
	struct timeval cts;		/* completion timestamp */
	struct acl acl;
	int an;				/* argument count */
	char **av;			/* argument vector */
	char *comment;
};

#define LOGIN_SECS		15
#define AYT_SECS		(10*60)
#define SYNC_SECS		(30*60)
#define SELECT_SECS		(10*60)

/* Globals */
extern struct state state;
extern int debug;
extern int verbose;
extern int reload;
extern const char *prog;
extern struct s2v astate2str[];
extern struct s2v rstate2str[];
extern const char version[];
extern const char *configfn;

/* Functions */
void appendreq(struct req **, struct req *) ;
void freereq(struct req *);
void intlistadd(struct cf *, const char *);
void intlistfree(struct intlist *);
void nbio(int);
