#!/bin/sh
# @(#) $Id: createrootca.sh,v 1.4 2004/03/03 03:09:30 leres Exp $ (LBL)
#
# createrootca.sh - create a root certificate authority key and certificate
#
# usage:
#
usage="usage: ${prog} [-f] rootca.key rootca.pem serial"
#
#	-f - overwrite existing key and cert
#
# This script creates a root certificate authority key and certificate.
# It will refuse to overwrite an existing key unless -f is specified.
#

PATH="/usr/ssl/bin:/usr/local/ssl/bin:${PATH}"
export PATH

prog="`basename $0`"

t1=/tmp/${prog}.1.$$

trap 'rm -f ${t1}; exit 1' 1 2 3 15

PRIVATEUMASK="umask 077"

force=0

set -- `getopt f $*`

for a do
	case "${a}" in

	-f)
		force=1
		shift
		;;

	--)
		shift
		break
		;;
	esac
done

if [ $# -ne 3 ]; then
	echo "${usage}" 1>&2
	exit 1
fi

key=$1
pem=$2
serial=$3
privatedir="`dirname ${key}`"
certdir="`dirname ${pem}`"

if [ ${force} -eq 0 ]; then
	if [ -f ${key} ]; then
		echo "${prog}: ${key} already exists (aborting)" 1>&2
		exit 1
	fi
	if [ -f ${pem} ]; then
		echo "${prog}: ${pem} already exists (aborting)" 1>&2
		exit 1
	fi
	if [ -f ${serial} ]; then
		echo "${prog}: ${serial} already exists (aborting)" 1>&2
		exit 1
	fi
fi

if [ "${privatedir}" != "." -a ! -d ${privatedir} ]; then
	(${PRIVATEUMASK} ; mkdir ${privatedir})
	if [ $? -ne 0 ]; then
		echo "${prog}: (aborting)" 1>&2
		exit 1
	fi
fi

echo ">> Create a key for the root certificate authority"
(${PRIVATEUMASK} ; set -x ; openssl genrsa -out ${key} 1024)
if [ $? -ne 0 ]; then
	exit 1
fi

if [ "${certdir}" != "." -a ! -d ${certdir} ]; then
	mkdir ${certdir}
	if [ $? -ne 0 ]; then
		echo "${prog}: (aborting)" 1>&2
		exit 1
	fi
fi

cat <<EOF >${t1}
[ req ]
prompt				= no
default_bits			= 1024
default_keyfile			= ${key}
distinguished_name		= req_distinguished_name
x509_extensions			= v3_ca
string_mask			= nombstr
req_extensions			= v3_req

[ req_distinguished_name ]
countryName			= ZZ
stateOrProvinceName		= Atlantis
localityName			= The Village
0.organizationName		= Unsuitable for security at any keysize
organizationalUnitName		= Bogon Anonymous Certificate Authority
commonName			= acl2
emailAddress			= root@`hostname`

[ v3_ca ]
basicConstraints		= critical,CA:true
subjectKeyIdentifier		= hash

[ v3_req ]
nsCertType			= objsign,server
EOF


echo ">> Self sign the the root certificate authority"
openssl req -new -x509 -nodes -days 3650 -config ${t1} -key ${key} -out ${pem}
if [ $? -ne 0 ]; then
	exit 1
fi

if [ ! -s ${serial} ]; then
	echo "01" > ${serial}
fi

echo ">> Successful!"

rm -f ${t1}
