/*
 * Copyright (c) 2004
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that: (1) source code distributions
 * retain the above copyright notice and this paragraph in its entirety, (2)
 * distributions including binary code include the above copyright notice and
 * this paragraph in its entirety in the documentation or other materials
 * provided with the distribution, and (3) all advertising materials mentioning
 * features or use of this software display the following acknowledgement:
 * ``This product includes software developed by the University of California,
 * Lawrence Berkeley Laboratory and its contributors.'' Neither the name of
 * the University nor the names of its contributors may be used to endorse
 * or promote products derived from this software without specific prior
 * written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef lint
static const char rcsid[] =
    "@(#) $Id: io.c,v 1.1 2004/05/12 02:18:35 leres Exp $ (LBL)";
#endif

#include <sys/types.h>
#include <sys/time.h>

#ifdef HAVE_MALLOC_H
#include <malloc.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <unistd.h>

#include "acld.h"
#include "util.h"

void
iofree(register struct iobuf *ip)
{
	if (ip->buf != NULL) {
		free(ip->buf);
		ip->buf = NULL;
	}
	ip->len = 0;
	ip->size = 0;
}

/* Return a line (newline terminated) or partial line (zero terminated) */
char *
iogetstr(register struct iobuf *ip)
{
	register char *cp;
	register size_t size;
	static char *buf = NULL;

	if (buf != NULL) {
		free(buf);
		buf = NULL;
	}
	if (ip->len == 0)
		return (NULL);

	for (cp = ip->buf; *cp != '\n' && *cp != '\0'; ++cp)
		continue;

	size = cp - ip->buf + 1;
	buf = new(size + 1, sizeof(*buf), "iogetstr buf");
	strncpy(buf, ip->buf, size);
	buf[size] = '\0';
	if (*cp == '\0')
		ip->len = 0;
	else {
		memmove(ip->buf, ip->buf + size, ip->len - size + 1);
		ip->len -= size;
	}
	ip->buf[ip->len] = '\0';
	return (buf);
}

/* Return true if there's a newline in the buffer */
int
iohaveline(register struct iobuf *ip)
{
	register int n;
	register char *cp;

	for (n = ip->len, cp = ip->buf; n > 0; --n, ++cp)
		if (*cp == '\n')
			return (1);
	return (0);
}

/* Read from a socket and to an iobuf */
int
ioread(register int fd, register struct iobuf *ip)
{
	register ssize_t cc;

	dynarray((void **)&ip->buf, &ip->len, &ip->size,
	    sizeof(*ip->buf), 8192, 8192, "ioread buf");

	cc = read(fd, ip->buf + ip->len, ip->size - ip->len - 1);
	if (cc < 0)
		return (cc);

	ip->len += cc;
	ip->buf[ip->len] = '\0';
	return (ip->len);
}

/* Write from an iobuf to a socket */
int
iowrite(register int fd, register struct iobuf *ip)
{
	register ssize_t cc;

	cc = write(fd, ip->buf, ip->len);
	if (cc < 0) {
		/* Bummer */
		ip->len = 0;
		return (cc);
	}
	if (cc > ip->len) {
		log(LOG_ERR, "iowrite: cc > ip-len, %d > %d (Can't happen?)\n",
		    cc, ip->len);
		cc = ip->len;
	}
	if (cc == ip->len) {
		ip->len = 0;
		return (cc);
	}

	/* Move leftover chars on over */
	memmove(ip->buf, ip->buf + cc, ip->len - cc);
	ip->len -= cc;
	return (cc);
}
